@@ -35,6 +35,8 @@ from reader import InvalidSearchQueryError
 from reader import ParseError
 from reader import ReaderError
 from reader._plugins import Loader
+from reader.types import _get_entry_content
+
 
 blueprint = Blueprint('reader', __name__)
 
@@ -47,6 +49,26 @@ def humanize_naturaltime(dt):
     return humanize.naturaltime(dt, when=when)
 
 
+@blueprint.app_template_filter()
+def humanize_apnumber(value):
+    return humanize.apnumber(value)
+
+
+@blueprint.app_template_filter()
+def toyaml(data):
+    return yaml.safe_dump(data)
+
+
+@blueprint.app_template_global()
+def debug_maxrss_mib():
+    import resource
+    import sys
+
+    return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 2 ** (
+        20 if sys.platform == 'darwin' else 10
+    )
+
+
 @blueprint.app_template_filter()
 def log_scale(n, p=2):
     # https://github.com/lemon24/reader/issues/249#issuecomment-893440484
@@ -69,6 +91,7 @@ def get_reader():
         g.reader = current_app.config['READER_CONFIG'].make_reader(
             'app', plugin_loader=current_app.plugin_loader
         )
+        # TODO: lowering g.reader._storage.chunk_size may reduce memory usage slightly
     return g.reader
 
 
@@ -139,6 +162,9 @@ class EntryProxy:
                 rv.append(Content(highlighted(highlight), 'text/html'))
         return rv
 
+    def get_content(self, prefer_summary=False):
+        return _get_entry_content(self, prefer_summary)
+
 
 @dataclass(frozen=True)
 class FeedProxy:
@@ -156,6 +182,30 @@ class FeedProxy:
         return self._entry.feed.title
 
 
+@dataclass
+class ResourceTags:
+    """Represent a bunch of tags in a reserved-name-scheme-agnostic way."""
+
+    reader: dict
+    # plugin: dict
+    # user: dict
+
+
+ENTRY_TAGS_READER = ['readtime']
+
+
+def get_entry_tags(reader, entry):
+    missing = object()
+
+    reader_tags = {}
+    for key in ENTRY_TAGS_READER:
+        value = reader.get_tag(entry, reader.make_reader_reserved_name(key), missing)
+        if value is not missing:
+            reader_tags[key] = value
+
+    return ResourceTags(reader=reader_tags)
+
+
 @blueprint.route('/')
 def entries():
     show = request.args.get('show', 'unread')
@@ -183,7 +233,7 @@ def entries():
         feed = reader.get_feed(feed_url, None)
         if not feed:
             abort(404)
-        feed_tags = list(reader.get_feed_tags(feed))
+        feed_tags = list(reader.get_tag_keys(feed))
 
     args = request.args.copy()
     query = args.pop('q', None)
@@ -245,8 +295,7 @@ def entries():
     counts = get_entry_counts(**kwargs) if with_counts else None
 
     try:
-        first = next(entries)
-        entries = itertools.chain([first], entries)
+        entries = itertools.chain([next(entries)], entries)
     except StopIteration:
         pass
     except InvalidSearchQueryError as e:
@@ -258,11 +307,14 @@ def entries():
         else:
             raise
 
-    entries = list(entries)
-
     entries_data = None
     if feed_url:
-        entries_data = [e.id for e in entries]
+        # TODO: duplicated from entries = ... above
+        # TODO: can be made faster with a get_entry_ids() method
+        entries_data = [
+            e.id
+            for e in get_entries(**kwargs, limit=request.args.get('limit', type=int))
+        ]
 
     feed_entry_counts = None
     if feed_url:
@@ -274,9 +326,11 @@ def entries():
     # https://github.com/lemon24/reader/issues/81
     get_flashed_messages()
 
+    entries_and_tags = ((e, get_entry_tags(reader, e)) for e in entries)
+
     return stream_template(
         'entries.html',
-        entries=entries,
+        entries_and_tags=entries_and_tags,
         feed=feed,
         feed_tags=feed_tags,
         entries_data=entries_data,
@@ -316,10 +370,12 @@ def preview():
     entries = list(reader.get_entries())
     feed_entry_counts = reader.get_entry_counts(feed=url)
 
+    entries_and_tags = ((e, get_entry_tags(reader, e)) for e in entries)
+
     # TODO: maybe limit
     return stream_template(
         'entries.html',
-        entries=entries,
+        entries_and_tags=entries_and_tags,
         feed=feed,
         read_only=True,
         feed_entry_counts=feed_entry_counts,
@@ -341,6 +397,19 @@ def add_entry():
     )
 
 
+FEED_SORT_NATIVE = {'title', 'added'}
+FEED_SORT_FANCY = {
+    'important': lambda counts: counts.important,
+    # TODO: an unread/unimportant property would be nice
+    'unread': lambda counts: counts.total - counts.read,
+    # TODO: if we keep these average intervals, properties for them might be nice too
+    'avg1m': lambda counts: counts.averages[0],
+    'avg3m': lambda counts: counts.averages[1],
+    'avg1y': lambda counts: counts.averages[2],
+}
+FEED_SORT_ALL = FEED_SORT_NATIVE.union(FEED_SORT_FANCY)
+
+
 @blueprint.route('/feeds')
 def feeds():
     broken = request.args.get('broken')
@@ -350,7 +419,7 @@ def feeds():
     updates_enabled = {None: None, 'no': False, 'yes': True}[updates_enabled]
 
     sort = request.args.get('sort', 'title')
-    assert sort in ('title', 'added')
+    assert sort in FEED_SORT_ALL
 
     error = None
 
@@ -376,19 +445,10 @@ def feeds():
 
     with_counts = request.args.get('counts')
     with_counts = {None: None, 'no': False, 'yes': True}[with_counts]
-    counts = reader.get_feed_counts(**kwargs) if with_counts else None
 
-    feed_data = []
+    counts = None
     try:
-        feeds = reader.get_feeds(sort=sort, **kwargs)
-        feed_data = (
-            (
-                feed,
-                list(reader.get_feed_tags(feed)),
-                reader.get_entry_counts(feed=feed) if with_counts else None,
-            )
-            for feed in feeds
-        )
+        counts = reader.get_feed_counts(**kwargs) if with_counts else None
     except ValueError as e:
         # TODO: there should be a better way of matching this kind of error
         if 'tag' in str(e).lower():
@@ -396,6 +456,45 @@ def feeds():
         else:
             raise
 
+    if sort in FEED_SORT_NATIVE:
+        kwargs['sort'] = sort
+
+    feeds = reader.get_feeds(**kwargs)
+    try:
+        feeds = itertools.chain([next(feeds)], feeds)
+    except StopIteration:
+        pass
+    except ValueError as e:
+        # TODO: there should be a better way of matching this kind of error
+        if 'tag' in str(e).lower():
+            error = f"invalid tag query: {e}: {tags_str}"
+        else:
+            raise
+
+    feed_data = (
+        (
+            feed,
+            list(reader.get_tag_keys(feed)),
+            reader.get_entry_counts(feed=feed)
+            if with_counts or sort in FEED_SORT_FANCY
+            else None,
+        )
+        for feed in feeds
+    )
+
+    # TODO: it would be nice if get_feeds() did this for us (streaming too)
+    if sort in FEED_SORT_FANCY:
+
+        def fancy_sort_key(data):
+            feed, tags, counts = data
+            return (
+                -FEED_SORT_FANCY[sort](counts),
+                feed.user_title or feed.title or '',
+                feed.url,
+            )
+
+        feed_data = sorted(feed_data, key=fancy_sort_key)
+
     # Ensure flashed messages get removed from the session.
     # https://github.com/lemon24/reader/issues/81
     get_flashed_messages()
@@ -409,12 +508,22 @@ def feeds():
 def metadata():
     reader = get_reader()
 
-    feed_url = request.args['feed']
-    feed = reader.get_feed(feed_url, None)
-    if not feed:
-        abort(404)
+    feed = entry = None
+    if 'feed' in request.args:
+        if 'entry' not in request.args:
+            resource_id = request.args['feed']
+            feed = reader.get_feed(resource_id, None)
+        else:
+            resource_id = request.args['feed'], request.args['entry']
+            entry = reader.get_entry(resource_id, None)
+            if entry:
+                feed = entry.feed
+        if not feed:
+            abort(404)
+    else:
+        resource_id = ()
 
-    metadata = reader.get_feed_metadata(feed_url)
+    metadata = reader.get_tags(resource_id)
 
     # Ensure flashed messages get removed from the session.
     # https://github.com/lemon24/reader/issues/81
@@ -423,6 +532,7 @@ def metadata():
     return stream_template(
         'metadata.html',
         feed=feed,
+        entry=entry,
         metadata=metadata,
         to_pretty_json=lambda t: yaml.safe_dump(t),
     )
@@ -439,7 +549,9 @@ def entry():
     if not entry:
         abort(404)
 
-    return render_template('entry.html', entry=entry)
+    tags = get_entry_tags(reader, entry)
+
+    return render_template('entry.html', entry=entry, tags=tags)
 
 
 @blueprint.route('/tags')
@@ -450,7 +562,7 @@ def tags():
     with_counts = {None: None, 'no': False, 'yes': True}[with_counts]
 
     def iter_tags():
-        for tag in itertools.chain([None, True, False], reader.get_feed_tags()):
+        for tag in itertools.chain([None, True, False], reader.get_tag_keys((None,))):
             feed_counts = None
             entry_counts = None
 
@@ -569,32 +681,43 @@ def update_feed_title(data):
     get_reader().set_feed_user_title(feed_url, feed_title)
 
 
+def _resource_id_from_data(data):
+    rv = ()
+    if 'feed-url' in data:
+        rv += (data['feed-url'],)
+        if 'entry-id' in data:
+            rv += (data['entry-id'],)
+    return rv
+
+
 @form_api
 @readererror_to_apierror()
 def add_metadata(data):
-    feed_url = data['feed-url']
+    resource_id = _resource_id_from_data(data)
     key = data['key']
-    get_reader().set_feed_metadata_item(feed_url, key, None)
+    get_reader().set_tag(resource_id, key, None)
 
 
 @form_api
 @readererror_to_apierror()
 def update_metadata(data):
-    feed_url = data['feed-url']
+    resource_id = _resource_id_from_data(data)
     key = data['key']
     try:
         value = yaml.safe_load(data['value'])
     except yaml.YAMLError as e:
-        raise APIError("invalid JSON: {}".format(e), (feed_url, key)) from e
-    get_reader().set_feed_metadata_item(feed_url, key, value)
+        raise APIError("invalid JSON: {}".format(e), resource_id + (key,)) from e
+    get_reader().set_tag(resource_id, key, value)
 
 
-@form_api(really=True)
+# TODO: @form_api(really=True)
+@form_api
 @readererror_to_apierror()
 def delete_metadata(data):
-    feed_url = data['feed-url']
+    resource_id = _resource_id_from_data(data)
+    print(resource_id)
     key = data['key']
-    get_reader().delete_feed_metadata_item(feed_url, key)
+    get_reader().delete_tag(resource_id, key)
 
 
 @form_api
@@ -604,12 +727,12 @@ def update_feed_tags(data):
     feed_tags = set(data['feed-tags'].split())
 
     reader = get_reader()
-    tags = set(reader.get_feed_tags(feed_url))
+    tags = set(reader.get_tag_keys(feed_url))
 
     for tag in tags - feed_tags:
-        reader.remove_feed_tag(feed_url, tag)
+        reader.delete_tag(feed_url, tag, True)
     for tag in feed_tags - tags:
-        reader.add_feed_tag(feed_url, tag)
+        reader.set_tag(feed_url, tag)
 
 
 @form_api(really=True)
