@@ -2,33 +2,40 @@ import calendar
 import json
 import logging
 import mimetypes
+import pathlib
+import shutil
+import tempfile
 import time
+import warnings
 from collections import OrderedDict
 from contextlib import contextmanager
+from contextlib import nullcontext
 from dataclasses import dataclass
 from datetime import datetime
 from datetime import timezone
 from typing import Any
-from typing import BinaryIO
+from typing import Callable
 from typing import cast
+from typing import Collection
 from typing import ContextManager
 from typing import Dict
+from typing import Generic
+from typing import IO
 from typing import Iterable
 from typing import Iterator
 from typing import List
 from typing import Mapping
 from typing import NamedTuple
 from typing import Optional
+from typing import Protocol
+from typing import runtime_checkable
 from typing import Tuple
 from typing import Type
 from typing import TypeVar
 from typing import Union
 
-import feedparser  # type: ignore
-import iso8601  # type: ignore
+import iso8601
 import requests
-from typing_extensions import Protocol
-from typing_extensions import runtime_checkable
 
 import reader
 from ._http_utils import parse_accept_header
@@ -39,14 +46,18 @@ from ._requests_utils import SessionWrapper
 from ._requests_utils import TimeoutHTTPAdapter
 from ._requests_utils import TimeoutType
 from ._types import EntryData
+from ._types import EntryForUpdate
 from ._types import FeedData
+from ._types import FeedForUpdate
 from ._types import ParsedFeed
 from ._url_utils import extract_path
-from ._url_utils import is_windows_device_file
 from ._url_utils import normalize_url
 from ._url_utils import resolve_root
+from ._utils import MapType
+from ._vendor import feedparser
 from .exceptions import InvalidFeedURLError
 from .exceptions import ParseError
+from .types import _namedtuple_compat
 from .types import Content
 from .types import Enclosure
 
@@ -56,23 +67,34 @@ log = logging.getLogger('reader')
 
 Headers = Mapping[str, str]
 
+T_co = TypeVar('T_co', covariant=True)
+T_cv = TypeVar('T_cv', contravariant=True)
 
-class RetrieveResult(NamedTuple):
-    file: BinaryIO
+
+# RetrieveResult was a NamedTuple, but generic ones aren't supported yet:
+# https://github.com/python/mypy/issues/685
+
+
+@dataclass(frozen=True)
+class RetrieveResult(_namedtuple_compat, Generic[T_co]):
+    file: T_co
     mime_type: Optional[str] = None
     http_etag: Optional[str] = None
     http_last_modified: Optional[str] = None
     headers: Optional[Headers] = None
 
 
-class RetrieverType(Protocol):
+class RetrieverType(Protocol[T_co]):  # pragma: no cover
+
+    slow_to_read: bool
+
     def __call__(
         self,
         url: str,
         http_etag: Optional[str],
         http_last_modified: Optional[str],
         http_accept: Optional[str],
-    ) -> ContextManager[Optional[RetrieveResult]]:  # pragma: no cover
+    ) -> ContextManager[Optional[RetrieveResult[T_co]]]:
         ...
 
     def validate_url(self, url: str) -> None:
@@ -84,24 +106,61 @@ class RetrieverType(Protocol):
         """
 
 
-# TODO: use Collection instead of Iterable
-FeedAndEntries = Tuple[FeedData, Iterable[EntryData]]
+@runtime_checkable
+class FeedForUpdateRetrieverType(RetrieverType[T_co], Protocol):  # pragma: no cover
+    def process_feed_for_update(self, feed: FeedForUpdate) -> FeedForUpdate:
+        ...
 
 
-class ParserType(Protocol):
+FeedAndEntries = Tuple[FeedData, Collection[EntryData]]
+EntryPair = Tuple[EntryData, Optional[EntryForUpdate]]
+
+
+class ParserType(Protocol[T_cv]):  # pragma: no cover
     def __call__(
-        self, url: str, file: BinaryIO, headers: Optional[Headers]
-    ) -> FeedAndEntries:  # pragma: no cover
+        self, url: str, file: T_cv, headers: Optional[Headers]
+    ) -> FeedAndEntries:
         ...
 
 
 @runtime_checkable
-class AwareParserType(ParserType, Protocol):
+class HTTPAcceptParserType(ParserType[T_cv], Protocol):  # pragma: no cover
     @property
-    def http_accept(self) -> str:  # pragma: no cover
+    def http_accept(self) -> str:
         ...
 
 
+@runtime_checkable
+class EntryPairsParserType(ParserType[T_cv], Protocol):  # pragma: no cover
+    def process_entry_pairs(
+        self, url: str, pairs: Iterable[EntryPair]
+    ) -> Iterable[EntryPair]:
+        ...
+
+
+class FeedArgument(Protocol):  # pragma: no cover
+    @property
+    def url(self) -> str:
+        ...
+
+    @property
+    def http_etag(self) -> Optional[str]:
+        ...
+
+    @property
+    def http_last_modified(self) -> Optional[str]:
+        ...
+
+
+class FeedArgumentTuple(NamedTuple):
+    url: str
+    http_etag: Optional[str] = None
+    http_last_modified: Optional[str] = None
+
+
+FA = TypeVar('FA', bound=FeedArgument)
+
+
 SESSION_TIMEOUT = (3.05, 60)
 
 
@@ -111,7 +170,7 @@ def default_parser(
     parser = Parser()
     parser.session_timeout = session_timeout
 
-    http_retriever = HTTPRetriever(parser.make_session)
+    http_retriever = HTTPRetriever(parser.get_session)
     parser.mount_retriever('https://', http_retriever)
     parser.mount_retriever('http://', http_retriever)
     if feed_root is not None:
@@ -137,11 +196,67 @@ class Parser:
     )
 
     def __init__(self) -> None:
-        self.retrievers: 'OrderedDict[str, RetrieverType]' = OrderedDict()
-        self.parsers_by_mime_type: Dict[str, List[Tuple[float, ParserType]]] = {}
-        self.parsers_by_url: Dict[str, ParserType] = {}
+        # Typing the link between parser and retriever would be nice,
+        # but seems hard to do; for a simplified version of the problem:
+        # https://gist.github.com/lemon24/b9338bea9aef176cbadcbfc25687dcf5
+        #
+        # Higher Kinded Types might be a way of doing it,
+        # https://returns.readthedocs.io/en/latest/pages/hkt.html
+
+        self.retrievers: 'OrderedDict[str, RetrieverType[Any]]' = OrderedDict()
+        self.parsers_by_mime_type: Dict[str, List[Tuple[float, ParserType[Any]]]] = {}
+        self.parsers_by_url: Dict[str, ParserType[Any]] = {}
         self.session_hooks = SessionHooks()
         self.session_timeout: TimeoutType = None
+        self.session: Optional[SessionWrapper] = None
+
+    def parallel(
+        self, feeds: Iterable[FA], map: MapType = map, is_parallel: bool = True
+    ) -> Iterable[Tuple[FA, Union[Optional[ParsedFeed], ParseError]]]:
+        def retrieve(
+            feed: FA,
+        ) -> Tuple[FA, Union[ContextManager[Optional[RetrieveResult[Any]]], Exception]]:
+            try:
+                context = self.retrieve(
+                    feed.url, feed.http_etag, feed.http_last_modified, is_parallel
+                )
+                return feed, context
+            except Exception as e:
+                # pass around *all* exception types,
+                # unhandled exceptions get swallowed by the thread otherwise
+                log.debug("retrieve() exception, traceback follows", exc_info=True)
+                return feed, e
+
+        with self.persistent_session():
+
+            # if stuff hangs weirdly during debugging, change this to builtins.map
+            retrieve_results = map(retrieve, feeds)
+
+            # we could parallelize parse() as well;
+            # however, most of the time is spent in pure-Python code,
+            # which doesn't benefit from the threads on CPython:
+            # https://github.com/lemon24/reader/issues/261#issuecomment-956412131
+
+            for feed, context in retrieve_results:
+                if isinstance(context, ParseError):
+                    yield feed, context
+                    continue
+
+                if isinstance(context, Exception):  # pragma: no cover
+                    raise context
+
+                try:
+                    with context as result:
+
+                        if not result or isinstance(result, ParseError):
+                            yield feed, result
+                            continue
+
+                        yield feed, self.parse(feed.url, result)
+
+                except ParseError as e:
+                    log.debug("parse() exception, traceback follows", exc_info=True)
+                    yield feed, e
 
     def __call__(
         self,
@@ -149,6 +264,25 @@ class Parser:
         http_etag: Optional[str] = None,
         http_last_modified: Optional[str] = None,
     ) -> Optional[ParsedFeed]:
+        """Thin wrapper for parallel(), to be used by parser tests."""
+
+        feed = FeedArgumentTuple(url, http_etag, http_last_modified)
+
+        # is_parallel=True ensures the parser tests cover more code
+        ((_, result),) = self.parallel([feed], is_parallel=True)
+
+        if isinstance(result, Exception):
+            raise result
+        return result
+
+    def retrieve(
+        self,
+        url: str,
+        http_etag: Optional[str] = None,
+        http_last_modified: Optional[str] = None,
+        is_parallel: bool = False,
+    ) -> ContextManager[Optional[RetrieveResult[Any]]]:
+
         parser = self.get_parser_by_url(url)
 
         http_accept: Optional[str]
@@ -164,38 +298,69 @@ class Parser:
             http_accept = None
 
         retriever = self.get_retriever(url)
+        context = retriever(url, http_etag, http_last_modified, http_accept)
 
-        with retriever(url, http_etag, http_last_modified, http_accept) as result:
+        if not (is_parallel and retriever.slow_to_read):
+            return context
+
+        # Ensure we read everything *before* yielding the response,
+        # i.e. __enter__() does most of the work.
+        #
+        # Gives a ~20% speed improvement over yielding response.raw
+        # when updating many feeds in parallel,
+        # with a 2-8% increase in memory usage:
+        # https://github.com/lemon24/reader/issues/261#issuecomment-956303210
+        #
+        # SpooledTemporaryFile() is just as fast as TemporaryFile():
+        # https://github.com/lemon24/reader/issues/261#issuecomment-957469041
+
+        with context as result:
             if not result:
-                return None
+                return nullcontext()
 
+            temp = tempfile.TemporaryFile()
+            shutil.copyfileobj(result.file, temp)
+            temp.seek(0)
+
+            result = result._replace(file=temp)
+
+        @contextmanager
+        def make_context() -> Iterator[RetrieveResult[Any]]:
+            assert result is not None, result  # for mypy
+            with wrap_exceptions(url, "while reading feed"), temp:
+                yield result
+
+        return make_context()
+
+    def parse(self, url: str, result: RetrieveResult[Any]) -> ParsedFeed:
+        parser, mime_type = self.get_parser(url, result.mime_type)
+        feed, entries = parser(url, result.file, result.headers)
+        return ParsedFeed(
+            feed, entries, result.http_etag, result.http_last_modified, mime_type
+        )
+
+    def get_parser(
+        self, url: str, mime_type: Optional[str]
+    ) -> Tuple[ParserType[Any], Optional[str]]:
+        parser = self.get_parser_by_url(url)
+        if not parser:
+            if not mime_type:
+                mime_type, _ = mimetypes.guess_type(url)
+
+            # https://tools.ietf.org/html/rfc7231#section-3.1.1.5
+            #
+            # > If a Content-Type header field is not present, the recipient
+            # > MAY either assume a media type of "application/octet-stream"
+            # > ([RFC2046], Section 4.5.1) or examine the data to determine its type.
+            #
+            if not mime_type:
+                mime_type = 'application/octet-stream'
+
+            parser = self.get_parser_by_mime_type(mime_type)
             if not parser:
-                mime_type = result.mime_type
-                if not mime_type:
-                    mime_type, _ = mimetypes.guess_type(url)
+                raise ParseError(url, message=f"no parser for MIME type {mime_type!r}")
 
-                # https://tools.ietf.org/html/rfc7231#section-3.1.1.5
-                #
-                # > If a Content-Type header field is not present, the recipient
-                # > MAY either assume a media type of "application/octet-stream"
-                # > ([RFC2046], Section 4.5.1) or examine the data to determine its type.
-                #
-                if not mime_type:
-                    mime_type = 'application/octet-stream'
-
-                parser = self.get_parser_by_mime_type(mime_type)
-                if not parser:
-                    raise ParseError(
-                        url, message=f"no parser for MIME type {mime_type!r}"
-                    )
-
-            feed, entries = parser(
-                url,
-                result.file,
-                result.headers,
-            )
-
-        return ParsedFeed(feed, entries, result.http_etag, result.http_last_modified)
+        return parser, mime_type
 
     def validate_url(self, url: str) -> None:
         """Check if ``url`` is valid without actually retrieving it.
@@ -213,23 +378,23 @@ class Parser:
         except ValueError as e:
             raise InvalidFeedURLError(url) from e
 
-    def mount_retriever(self, prefix: str, retriever: RetrieverType) -> None:
+    def mount_retriever(self, prefix: str, retriever: RetrieverType[Any]) -> None:
         self.retrievers[prefix] = retriever
         keys_to_move = [k for k in self.retrievers if len(k) < len(prefix)]
         for key in keys_to_move:
             self.retrievers[key] = self.retrievers.pop(key)
 
-    def get_retriever(self, url: str) -> RetrieverType:
+    def get_retriever(self, url: str) -> RetrieverType[Any]:
         for prefix, retriever in self.retrievers.items():
             if url.lower().startswith(prefix.lower()):
                 return retriever
         raise ParseError(url, message="no retriever for URL")
 
     def mount_parser_by_mime_type(
-        self, parser: ParserType, http_accept: Optional[str] = None
+        self, parser: ParserType[Any], http_accept: Optional[str] = None
     ) -> None:
         if not http_accept:
-            if not isinstance(parser, AwareParserType):
+            if not isinstance(parser, HTTPAcceptParserType):
                 raise TypeError("unaware parser type with no http_accept given")
             http_accept = parser.http_accept
 
@@ -245,7 +410,7 @@ class Parser:
             index = existing_qualities[0][1] if existing_qualities else 0
             parsers.insert(index, (quality, parser))
 
-    def get_parser_by_mime_type(self, mime_type: str) -> Optional[ParserType]:
+    def get_parser_by_mime_type(self, mime_type: str) -> Optional[ParserType[Any]]:
         parsers = self.parsers_by_mime_type.get(mime_type, ())
         if not parsers:
             parsers = self.parsers_by_mime_type.get('*/*', ())
@@ -253,11 +418,11 @@ class Parser:
             return parsers[-1][1]
         return None
 
-    def mount_parser_by_url(self, url: str, parser: ParserType) -> None:
+    def mount_parser_by_url(self, url: str, parser: ParserType[Any]) -> None:
         url = normalize_url(url)
         self.parsers_by_url[url] = parser
 
-    def get_parser_by_url(self, url: str) -> Optional[ParserType]:
+    def get_parser_by_url(self, url: str) -> Optional[ParserType[Any]]:
         # we might change this to have some smarter matching, but YAGNI
         url = normalize_url(url)
         return self.parsers_by_url.get(url)
@@ -273,6 +438,40 @@ class Parser:
 
         return session
 
+    def get_session(self) -> ContextManager[SessionWrapper]:
+        if self.session:
+            return nullcontext(self.session)
+        return self.make_session()
+
+    @contextmanager
+    def persistent_session(self) -> Iterator[SessionWrapper]:
+        # note: this is NOT threadsafe, but is reentrant
+
+        if self.session:  # pragma: no cover
+            yield self.session
+            return
+
+        with self.make_session() as session:
+            self.session = session
+            try:
+                yield session
+            finally:
+                self.session = None
+
+    def process_feed_for_update(self, feed: FeedForUpdate) -> FeedForUpdate:
+        retriever = self.get_retriever(feed.url)
+        if not isinstance(retriever, FeedForUpdateRetrieverType):
+            return feed
+        return retriever.process_feed_for_update(feed)
+
+    def process_entry_pairs(
+        self, url: str, mime_type: Optional[str], pairs: Iterable[EntryPair]
+    ) -> Iterable[EntryPair]:
+        parser, _ = self.get_parser(url, mime_type)
+        if not isinstance(parser, EntryPairsParserType):
+            return pairs
+        return parser.process_entry_pairs(url, pairs)
+
 
 @contextmanager
 def wrap_exceptions(url: str, when: str) -> Iterator[None]:
@@ -298,12 +497,16 @@ class FileRetriever:
 
     feed_root: str
 
+    slow_to_read = False
+
     def __post_init__(self) -> None:
         # give feed_root checks a chance to fail early
         self._normalize_url('known-good-feed-url')
 
     @contextmanager
-    def __call__(self, url: str, *args: Any, **kwargs: Any) -> Iterator[RetrieveResult]:
+    def __call__(
+        self, url: str, *args: Any, **kwargs: Any
+    ) -> Iterator[RetrieveResult[IO[bytes]]]:
         try:
             normalized_url = self._normalize_url(url)
         except ValueError as e:
@@ -320,17 +523,11 @@ class FileRetriever:
         path = extract_path(url)
         if self.feed_root:
             path = resolve_root(self.feed_root, path)
-            if is_windows_device_file(path):
-                raise ValueError("path must not be a device file")
+            if pathlib.PurePath(path).is_reserved():
+                raise ValueError("path must not be reserved")
         return path
 
 
-class _MakeSession(Protocol):
-    # https://github.com/python/mypy/issues/708#issuecomment-647124281 workaround
-    def __call__(self) -> SessionWrapper:  # pragma: no cover
-        ...
-
-
 @dataclass
 class HTTPRetriever:
 
@@ -358,7 +555,9 @@ class HTTPRetriever:
 
     """
 
-    make_session: _MakeSession
+    get_session: Callable[[], ContextManager[SessionWrapper]]
+
+    slow_to_read = True
 
     @contextmanager
     def __call__(
@@ -367,14 +566,12 @@ class HTTPRetriever:
         http_etag: Optional[str] = None,
         http_last_modified: Optional[str] = None,
         http_accept: Optional[str] = None,
-    ) -> Iterator[Optional[RetrieveResult]]:
+    ) -> Iterator[Optional[RetrieveResult[IO[bytes]]]]:
         request_headers = {}
         if http_accept:
             request_headers['Accept'] = http_accept
 
-        # TODO: maybe share the session in the parser?
-
-        with self.make_session() as session:
+        with self.get_session() as session:
             with wrap_exceptions(url, "while getting feed"):
                 response, http_etag, http_last_modified = _caching_get(
                     session,
@@ -414,9 +611,10 @@ class HTTPRetriever:
                 )
 
     def validate_url(self, url: str) -> None:
-        session = self.make_session().session
-        session.get_adapter(url)
-        session.prepare_request(requests.Request('GET', url))
+        with self.get_session() as session_wrapper:
+            session = session_wrapper.session
+            session.get_adapter(url)
+            session.prepare_request(requests.Request('GET', url))
 
 
 def _caching_get(
@@ -457,7 +655,7 @@ class FeedparserParser:
     def __call__(
         self,
         url: str,
-        file: BinaryIO,
+        file: IO[bytes],
         headers: Optional[Headers] = None,
     ) -> FeedAndEntries:
         """Like feedparser.parse(), but return a feed and entries,
@@ -469,7 +667,7 @@ class FeedparserParser:
         # feedparser content sanitization and relative link resolution should be ON.
         # https://github.com/lemon24/reader/issues/125
         # https://github.com/lemon24/reader/issues/157
-        result = feedparser.parse(
+        result = feedparser.parse(  # type: ignore[attr-defined]
             file,
             resolve_relative_uris=True,
             sanitize_html=True,
@@ -498,38 +696,47 @@ def _process_feedparser_dict(url: str, d: Any) -> FeedAndEntries:
         raise ParseError(url, message="unknown feed type")
 
     is_rss = d.version.startswith('rss')
-    updated, _ = _get_updated_published(d.feed, is_rss)
 
     feed = FeedData(
         url,
-        updated,
+        _get_datetime_fp_attr(d.feed, 'updated_parsed'),
         d.feed.get('title'),
         d.feed.get('link'),
         d.feed.get('author'),
         d.feed.get('subtitle') or None,
         d.version,
     )
-    # This must be a list, not a generator expression,
+
+    # entries must be a list, not a generator expression,
     # otherwise the user may get a ParseError when calling
     # next(parse_result.entries), i.e. after parse() returned.
-    entries = [_feedparser_entry(url, e, is_rss) for e in d.entries]
+    entries = []
+    first_parse_error = None
+
+    for e in d.entries:
+        try:
+            entry = _feedparser_entry(url, e, is_rss)
+        except ParseError as e:
+            # Skip entries that raise ParseError with a warning.
+            # https://github.com/lemon24/reader/issues/281
+            warnings.warn(e)
+            if not first_parse_error:
+                first_parse_error = e
+        else:
+            entries.append(entry)
+
+    # If all entries failed, raise the first exception.
+    if first_parse_error and not entries:
+        raise first_parse_error
 
     return feed, entries
 
 
-def _get_updated_published(
-    thing: Any, is_rss: bool
-) -> Tuple[Optional[datetime], Optional[datetime]]:
-    def convert(key: str) -> Any:
-        # feedparser.FeedParserDict.get('updated') defaults to published
-        # for historical reasons; "key in thing" bypasses that
-        value = thing[key] if key in thing else None
-        return _datetime_from_timetuple(value) if value else None
-
-    updated = convert('updated_parsed')
-    published = convert('published_parsed')
-
-    return updated, published
+def _get_datetime_fp_attr(thing: Any, key: str) -> Optional[datetime]:
+    # feedparser.FeedParserDict.get('updated') defaults to published
+    # for historical reasons; "key in thing" bypasses that
+    value = thing[key] if key in thing else None
+    return _datetime_from_timetuple(value) if value else None
 
 
 def _datetime_from_timetuple(tt: time.struct_time) -> datetime:
@@ -550,9 +757,7 @@ def _feedparser_entry(feed_url: str, entry: Any, is_rss: bool) -> EntryData:
         )
 
     if not id:
-        raise ParseError(feed_url, message="entry with no id or link fallback")
-
-    updated, published = _get_updated_published(entry, is_rss)
+        raise ParseError(feed_url, message="entry with no id or fallback")
 
     content = []
     for data in entry.get('content', ()):
@@ -575,11 +780,11 @@ def _feedparser_entry(feed_url: str, entry: Any, is_rss: bool) -> EntryData:
     return EntryData(
         feed_url,
         id,
-        updated,
+        _get_datetime_fp_attr(entry, 'updated_parsed'),
         entry.get('title'),
         entry.get('link'),
         entry.get('author'),
-        published,
+        _get_datetime_fp_attr(entry, 'published_parsed'),
         entry.get('summary'),
         tuple(content),
         tuple(enclosures),
@@ -595,7 +800,7 @@ class JSONFeedParser:
     def __call__(
         self,
         url: str,
-        file: BinaryIO,
+        file: IO[bytes],
         headers: Optional[Headers] = None,
     ) -> FeedAndEntries:
         try:
