@@ -4,18 +4,19 @@ import sqlite3
 from datetime import datetime
 from datetime import timedelta
 from functools import partial
-from itertools import chain
-from itertools import repeat
 from typing import Any
+from typing import cast
 from typing import Dict
 from typing import Iterable
 from typing import Mapping
 from typing import NamedTuple
 from typing import Optional
+from typing import overload
 from typing import Sequence
 from typing import Tuple
 from typing import Type
 from typing import TypeVar
+from typing import Union
 
 from ._sql_utils import BaseQuery
 from ._sql_utils import paginated_query
@@ -38,14 +39,13 @@ from ._utils import join_paginated_iter
 from ._utils import zero_or_one
 from .exceptions import EntryError
 from .exceptions import EntryExistsError
-from .exceptions import EntryMetadataNotFoundError
 from .exceptions import EntryNotFoundError
 from .exceptions import FeedExistsError
-from .exceptions import FeedMetadataNotFoundError
 from .exceptions import FeedNotFoundError
-from .exceptions import MetadataNotFoundError
 from .exceptions import ReaderError
 from .exceptions import StorageError
+from .exceptions import TagNotFoundError
+from .types import AnyResourceId
 from .types import Content
 from .types import Enclosure
 from .types import Entry
@@ -56,6 +56,9 @@ from .types import Feed
 from .types import FeedCounts
 from .types import FeedSortOrder
 from .types import JSONType
+from .types import MISSING
+from .types import MissingType
+from .types import ResourceId
 
 APPLICATION_ID = int(''.join(f'{ord(c):x}' for c in 'read'), 16)
 
@@ -69,8 +72,9 @@ _T = TypeVar('_T')
 def create_db(db: sqlite3.Connection) -> None:
     create_feeds(db)
     create_entries(db)
-    create_feed_metadata(db)
+    create_global_tags(db)
     create_feed_tags(db)
+    create_entry_tags(db)
     create_indexes(db)
 
 
@@ -147,10 +151,22 @@ def create_entries(db: sqlite3.Connection, name: str = 'entries') -> None:
     )
 
 
-def create_feed_metadata(db: sqlite3.Connection) -> None:
+def create_global_tags(db: sqlite3.Connection) -> None:
     db.execute(
         """
-        CREATE TABLE feed_metadata (
+        CREATE TABLE global_tags (
+            key TEXT NOT NULL,
+            value TEXT NOT NULL,
+            PRIMARY KEY (key)
+        );
+        """
+    )
+
+
+def create_feed_tags(db: sqlite3.Connection) -> None:
+    db.execute(
+        """
+        CREATE TABLE feed_tags (
             feed TEXT NOT NULL,
             key TEXT NOT NULL,
             value TEXT NOT NULL,
@@ -164,15 +180,17 @@ def create_feed_metadata(db: sqlite3.Connection) -> None:
     )
 
 
-def create_feed_tags(db: sqlite3.Connection) -> None:
+def create_entry_tags(db: sqlite3.Connection) -> None:
     db.execute(
         """
-        CREATE TABLE feed_tags (
+        CREATE TABLE entry_tags (
+            id TEXT NOT NULL,
             feed TEXT NOT NULL,
-            tag TEXT NOT NULL,
+            key TEXT NOT NULL,
+            value TEXT NOT NULL,
 
-            PRIMARY KEY (feed, tag),
-            FOREIGN KEY (feed) REFERENCES feeds(url)
+            PRIMARY KEY (id, feed, key),
+            FOREIGN KEY (id, feed) REFERENCES entries(id, feed)
                 ON UPDATE CASCADE
                 ON DELETE CASCADE
         );
@@ -184,14 +202,12 @@ class SchemaInfo(NamedTuple):
     table_prefix: str
     id_columns: Tuple[str, ...]
     not_found_exc: Type[ReaderError]
-    metadata_not_found_exc: Type[MetadataNotFoundError]
 
 
 SCHEMA_INFO = {
-    1: SchemaInfo('feed_', ('feed',), FeedNotFoundError, FeedMetadataNotFoundError),
-    2: SchemaInfo(
-        'entry_', ('feed', 'id'), EntryNotFoundError, EntryMetadataNotFoundError
-    ),
+    0: SchemaInfo('global_', (), ReaderError),
+    1: SchemaInfo('feed_', ('feed',), FeedNotFoundError),
+    2: SchemaInfo('entry_', ('feed', 'id'), EntryNotFoundError),
 }
 
 
@@ -327,15 +343,40 @@ def update_from_33_to_34(db: sqlite3.Connection) -> None:  # pragma: no cover
     recreate_search_triggers(db)
 
 
+def update_from_34_to_35(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # for https://github.com/lemon24/reader/issues/266
+    db.execute(
+        """
+        INSERT OR REPLACE INTO feed_metadata (feed, key, value)
+        SELECT feed, tag, coalesce((
+            SELECT value FROM feed_metadata AS fm
+            WHERE (fm.feed, fm.key) == (feed, tag)
+        ), 'null')
+        FROM feed_tags;
+        """
+    )
+    db.execute("DROP TABLE feed_tags;")
+    db.execute("ALTER TABLE feed_metadata RENAME TO feed_tags;")
+
+
+def update_from_35_to_36(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # for https://github.com/lemon24/reader/issues/272
+    create_global_tags(db)
+    create_entry_tags(db)
+
+
+# Row value support was added in 3.15.
+# TODO: Remove the Search.update() check once this gets bumped to >=3.18.
 MINIMUM_SQLITE_VERSION = (3, 15)
-REQUIRED_SQLITE_COMPILE_OPTIONS = ["ENABLE_JSON1"]
+# We use the JSON1 extension for entries.content.
+REQUIRED_SQLITE_FUNCTIONS = ['json_array_length']
 
 
 def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
     return setup_sqlite_db(
         db,
         create=create_db,
-        version=34,
+        version=36,
         migrations={
             # 1-9 removed before 0.1 (last in e4769d8ba77c61ec1fe2fbe99839e1826c17ace7)
             # 10-16 removed before 1.0 (last in 618f158ebc0034eefb724a55a84937d21c93c1a7)
@@ -345,13 +386,12 @@ def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
             31: update_from_31_to_32,
             32: update_from_32_to_33,
             33: update_from_33_to_34,
+            34: update_from_34_to_35,
+            35: update_from_35_to_36,
         },
         id=APPLICATION_ID,
-        # Row value support was added in 3.15.
-        # TODO: Remove the Search.update() check once this gets bumped to >=3.18.
         minimum_sqlite_version=MINIMUM_SQLITE_VERSION,
-        # We use the JSON1 extension for entries.content.
-        required_sqlite_compile_options=REQUIRED_SQLITE_COMPILE_OPTIONS,
+        required_sqlite_functions=REQUIRED_SQLITE_FUNCTIONS,
         wal_enabled=wal_enabled,
     )
 
@@ -580,9 +620,7 @@ class Storage:
     @wrap_exceptions_iter(StorageError)
     def get_feeds_for_update(
         self,
-        url: Optional[str] = None,
-        new: Optional[bool] = None,
-        enabled_only: bool = True,
+        filter_options: FeedFilterOptions = FeedFilterOptions(),  # noqa: B008
     ) -> Iterable[FeedForUpdate]:
         # Reader shouldn't care this is paginated,
         # so we don't expose any pagination stuff.
@@ -604,17 +642,10 @@ class Storage:
                 )
                 .FROM("feeds")
             )
-            context: Dict[str, object] = {}
 
             # TODO: stale and last_exception should be bool, not int
 
-            if url:
-                query.WHERE("url = :url")
-                context.update(url=url)
-            if new is not None:
-                query.WHERE(f"last_updated is {'' if new else 'NOT'} NULL")
-            if enabled_only:
-                query.WHERE("updates_enabled")
+            context = apply_feed_filter_options(query, filter_options)
 
             query.scrolling_window_order_by("url")
 
@@ -624,12 +655,26 @@ class Storage:
 
         yield from join_paginated_iter(inner, self.chunk_size)
 
-    def _get_entries_for_update_n_queries(
-        self, entries: Sequence[Tuple[str, str]]
+    def _get_entries_for_update(
+        self, entries: Iterable[Tuple[str, str]]
     ) -> Iterable[Optional[EntryForUpdate]]:
-        # We use an explicit transaction for speed
-        # (otherwise we get an implicit one for each query).
+        # We could fetch everything in a single query,
+        # but there are limits to the number of variables used in a query,
+        # and we'd have to fall back to this anyway.
+        # This is only ~10% slower than the single query version.
+        # Single query version last in e39b0134cb3a2fe2bb346d42355a764181926a82.
+
+        # This can't be a generator;
+        # we must get the result inside the transaction, otherwise we get:
+        #   Cursor needed to be reset because of commit/rollback
+        #   and can no longer be fetched from.
+        rv = []
+
         with self.db:
+            # We use an explicit transaction for speed,
+            # otherwise we get an implicit one for each query).
+            self.db.execute('BEGIN;')
+
             for feed_url, id in entries:  # noqa: B007
                 context = dict(feed_url=feed_url, id=id)
                 row = self.db.execute(
@@ -645,41 +690,9 @@ class Storage:
                     """,
                     context,
                 ).fetchone()
-                yield EntryForUpdate._make(row) if row else None
+                rv.append(EntryForUpdate._make(row) if row else None)
 
-    def _get_entries_for_update_one_query(
-        self, entries: Sequence[Tuple[str, str]]
-    ) -> Iterable[Optional[EntryForUpdate]]:
-        if not entries:  # pragma: no cover
-            return []
-
-        values_snippet = ', '.join(repeat('(?, ?)', len(entries)))
-        parameters = list(chain.from_iterable(entries))
-
-        rows = self.db.execute(
-            f"""
-            WITH
-                input(feed, id) AS (
-                    VALUES {values_snippet}
-                )
-            SELECT
-                entries.id IS NOT NULL,
-                entries.updated,
-                entries.published,
-                entries.data_hash,
-                entries.data_hash_changed
-            FROM input
-            LEFT JOIN entries
-                ON (input.id, input.feed) == (entries.id, entries.feed);
-            """,
-            parameters,
-        )
-
-        # This can't be a generator because we need to get OperationalError
-        # in this function (so get_entries_for_update() below can catch it).
-        return (
-            EntryForUpdate._make(rest) if exists else None for exists, *rest in rows
-        )
+        return rv
 
     @wrap_exceptions_iter(StorageError)
     def get_entries_for_update(
@@ -688,20 +701,9 @@ class Storage:
         # It's acceptable for this method to not be atomic. TODO: Why?
 
         iterables = chunks(self.chunk_size, entries) if self.chunk_size else (entries,)
+
         for iterable in iterables:
-
-            # There are two implementations for this method because
-            # the number of parameters in a query varies
-            # (e.g. 999 when compiling from sources, 250k on Ubuntu 18.04);
-            # https://github.com/lemon24/reader/issues/109
-            iterable = list(iterable)
-            try:
-                rv = self._get_entries_for_update_one_query(iterable)
-            except sqlite3.OperationalError as e:
-                if "too many SQL variables" not in str(e):
-                    raise
-                rv = self._get_entries_for_update_n_queries(iterable)
-
+            rv = self._get_entries_for_update(iterable)
             if self.chunk_size:
                 rv = list(rv)
             yield from rv
@@ -866,28 +868,6 @@ class Storage:
             )
         rowcount_exactly_one(cursor, lambda: FeedNotFoundError(url))
 
-    def _make_add_or_update_entries_args(
-        self, intent: EntryUpdateIntent
-    ) -> Mapping[str, Any]:
-        context = intent._asdict()
-        entry = context.pop('entry')
-        context.update(
-            entry._asdict(),
-            content=(
-                json.dumps([t._asdict() for t in entry.content])
-                if entry.content
-                else None
-            ),
-            enclosures=(
-                json.dumps([t._asdict() for t in entry.enclosures])
-                if entry.enclosures
-                else None
-            ),
-            data_hash=entry.hash,
-            data_hash_changed=context.pop('hash_changed'),
-        )
-        return context
-
     @wrap_exceptions(StorageError)
     def add_or_update_entries(self, entry_tuples: Iterable[EntryUpdateIntent]) -> None:
         iterables = (
@@ -904,105 +884,93 @@ class Storage:
             self._add_or_update_entries(iterable)
 
     def _add_or_update_entries(
-        self, entry_tuples: Iterable[EntryUpdateIntent], exclusive: bool = False
+        self, intents: Iterable[EntryUpdateIntent], exclusive: bool = False
     ) -> None:
-        # We need to capture the last value for exception handling
-        # (it's not guaranteed all the entries belong to the same feed).
-        # FIXME: In this case, is it ok to just fail other feeds too
-        # if we have an exception? If no, we should force the entries to
-        # belong to a single feed!
-        last_param: Mapping[str, Any] = {}
-
-        def make_params() -> Iterable[Mapping[str, Any]]:
-            nonlocal last_param
-            for last_param in map(self._make_add_or_update_entries_args, entry_tuples):
-                yield last_param
+        query = f"""
+            INSERT {'OR ABORT' if exclusive else 'OR REPLACE'} INTO entries (
+                id,
+                feed,
+                --
+                title,
+                link,
+                updated,
+                author,
+                published,
+                summary,
+                content,
+                enclosures,
+                read,
+                read_modified,
+                important,
+                important_modified,
+                last_updated,
+                first_updated,
+                first_updated_epoch,
+                feed_order,
+                original_feed,
+                data_hash,
+                data_hash_changed,
+                added_by
+            ) VALUES (
+                :id,
+                :feed_url,
+                :title,
+                :link,
+                :updated,
+                :author,
+                :published,
+                :summary,
+                :content,
+                :enclosures,
+                coalesce((
+                    SELECT read
+                    FROM entries
+                    WHERE id = :id AND feed = :feed_url
+                ), 0),
+                (
+                    SELECT read_modified
+                    FROM entries
+                    WHERE id = :id AND feed = :feed_url
+                ),
+                coalesce((
+                    SELECT important
+                    FROM entries
+                    WHERE id = :id AND feed = :feed_url
+                ), 0),
+                (
+                    SELECT important_modified
+                    FROM entries
+                    WHERE id = :id AND feed = :feed_url
+                ),
+                :last_updated,
+                coalesce(:first_updated, (
+                    SELECT first_updated
+                    FROM entries
+                    WHERE id = :id AND feed = :feed_url
+                )),
+                coalesce(:first_updated_epoch, (
+                    SELECT first_updated_epoch
+                    FROM entries
+                    WHERE id = :id AND feed = :feed_url
+                )),
+                :feed_order,
+                NULL, -- original_feed
+                :data_hash,
+                :data_hash_changed,
+                :added_by
+            );
+        """
 
         with self.db:
-
             try:
-                self.db.executemany(
-                    f"""
-                    INSERT {'OR ABORT' if exclusive else 'OR REPLACE'} INTO entries (
-                        id,
-                        feed,
-                        --
-                        title,
-                        link,
-                        updated,
-                        author,
-                        published,
-                        summary,
-                        content,
-                        enclosures,
-                        read,
-                        read_modified,
-                        important,
-                        important_modified,
-                        last_updated,
-                        first_updated,
-                        first_updated_epoch,
-                        feed_order,
-                        original_feed,
-                        data_hash,
-                        data_hash_changed,
-                        added_by
-                    ) VALUES (
-                        :id,
-                        :feed_url,
-                        :title,
-                        :link,
-                        :updated,
-                        :author,
-                        :published,
-                        :summary,
-                        :content,
-                        :enclosures,
-                        coalesce((
-                            SELECT read
-                            FROM entries
-                            WHERE id = :id AND feed = :feed_url
-                        ), 0),
-                        (
-                            SELECT read_modified
-                            FROM entries
-                            WHERE id = :id AND feed = :feed_url
-                        ),
-                        coalesce((
-                            SELECT important
-                            FROM entries
-                            WHERE id = :id AND feed = :feed_url
-                        ), 0),
-                        (
-                            SELECT important_modified
-                            FROM entries
-                            WHERE id = :id AND feed = :feed_url
-                        ),
-                        :last_updated,
-                        coalesce(:first_updated, (
-                            SELECT first_updated
-                            FROM entries
-                            WHERE id = :id AND feed = :feed_url
-                        )),
-                        coalesce(:first_updated_epoch, (
-                            SELECT first_updated_epoch
-                            FROM entries
-                            WHERE id = :id AND feed = :feed_url
-                        )),
-                        :feed_order,
-                        NULL, -- original_feed
-                        :data_hash,
-                        :data_hash_changed,
-                        :added_by
-                    );
-                    """,
-                    make_params(),
-                )
+                # we could use executemany(), but it's not noticeably faster
+                for intent in intents:
+                    self.db.execute(query, entry_update_intent_to_dict(intent))
+
             except sqlite3.IntegrityError as e:
                 e_msg = str(e).lower()
+                feed_url, entry_id = intent.entry.object_id
 
-                feed_url = last_param['feed_url']
-                entry_id = last_param['id']
                 log.debug(
                     "add_entry %r of feed %r: got IntegrityError",
                     entry_id,
@@ -1037,10 +1005,6 @@ class Storage:
         # (hopefully, there are both fewer entries to be deleted and
         # this takes less time per entry).
 
-        # I don't know how to check *which* entry did not exit
-        # with executemany(), so we use execute().
-        # TODO: Maybe add_or_update_entries() doesn't need executemany() either.
-
         delete_query = "DELETE FROM entries WHERE feed = :feed AND id = :id"
         added_by_query = "SELECT added_by FROM entries WHERE feed = :feed AND id = :id"
 
@@ -1161,31 +1125,46 @@ class Storage:
 
         return EntryCounts(*row[:4], row[4:7])  # type: ignore[call-arg]
 
-    def iter_metadata(
+    def get_tags(
         self,
-        object_id: Tuple[str, ...],
+        object_id: AnyResourceId,
         key: Optional[str] = None,
     ) -> Iterable[Tuple[str, JSONType]]:
         yield from join_paginated_iter(
-            partial(self.iter_metadata_page, object_id, key),
+            partial(self.get_tags_page, object_id, key),
             self.chunk_size,
         )
 
     @wrap_exceptions_iter(StorageError)
-    def iter_metadata_page(
+    def get_tags_page(
         self,
-        object_id: Tuple[str, ...],
+        object_id: AnyResourceId,
         key: Optional[str] = None,
         chunk_size: Optional[int] = None,
         last: Optional[_T] = None,
     ) -> Iterable[Tuple[Tuple[str, JSONType], Optional[_T]]]:
-        info = SCHEMA_INFO[len(object_id)]
+        query = Query().SELECT("key")
+        context: Dict[str, Any] = dict()
 
-        query = Query().SELECT("key", "value").FROM(f"{info.table_prefix}metadata")
+        if object_id is not None:
+            info = SCHEMA_INFO[len(object_id)]
+            query.FROM(f"{info.table_prefix}tags")
 
-        for column in info.id_columns:
-            query.WHERE(f"{column} = :{column}")
-        context = dict(zip(info.id_columns, object_id))
+            if not any(p is None for p in object_id):
+                query.SELECT("value")
+                for column in info.id_columns:
+                    query.WHERE(f"{column} = :{column}")
+                context.update(zip(info.id_columns, object_id))
+            else:
+                query.SELECT_DISTINCT("'null'")
+
+        else:
+            union = '\nUNION\n'.join(
+                f"SELECT key, value FROM {i.table_prefix}tags"
+                for i in SCHEMA_INFO.values()
+            )
+            query.WITH(('tags', union)).FROM('tags')
+            query.SELECT_DISTINCT("'null'")
 
         if key is not None:
             query.WHERE("key = :key")
@@ -1201,21 +1180,53 @@ class Storage:
             self.db, query, context, chunk_size, last, row_factory
         )
 
+    @overload
+    def set_tag(self, object_id: ResourceId, key: str) -> None:  # pragma: no cover
+        ...
+
+    @overload
+    def set_tag(
+        self, object_id: ResourceId, key: str, value: JSONType
+    ) -> None:  # pragma: no cover
+        ...
+
     @wrap_exceptions(StorageError)
-    def set_metadata(
-        self, object_id: Tuple[str, ...], key: str, value: JSONType
+    def set_tag(
+        self,
+        object_id: ResourceId,
+        key: str,
+        value: Union[MissingType, JSONType] = MISSING,
     ) -> None:
         info = SCHEMA_INFO[len(object_id)]
 
-        columns = info.id_columns + ('key', 'value')
+        params = dict(zip(info.id_columns, object_id), key=key)
+
+        id_columns = info.id_columns + ('key',)
+        id_columns_str = ', '.join(id_columns)
+        id_values_str = ', '.join(f':{c}' for c in id_columns)
+
+        if value is not MISSING:
+            value_str = ':value'
+            params.update(value=json.dumps(value))
+        else:
+            value_str = f"""
+                coalesce((
+                    SELECT value FROM {info.table_prefix}tags
+                    WHERE (
+                        {id_columns_str}
+                    ) == (
+                        {id_values_str}
+                    )
+                ), 'null')
+            """
+
         query = f"""
-            INSERT OR REPLACE INTO {info.table_prefix}metadata (
-                {', '.join(columns)}
+            INSERT OR REPLACE INTO {info.table_prefix}tags (
+                {id_columns_str}, value
             ) VALUES (
-                {', '.join(('?' for _ in columns))}
+                {id_values_str}, {value_str}
             )
         """
-        params = object_id + (key, json.dumps(value))
 
         with self.db:
             try:
@@ -1227,12 +1238,12 @@ class Storage:
                 raise info.not_found_exc(*object_id) from None
 
     @wrap_exceptions(StorageError)
-    def delete_metadata(self, object_id: Tuple[str, ...], key: str) -> None:
+    def delete_tag(self, object_id: ResourceId, key: str) -> None:
         info = SCHEMA_INFO[len(object_id)]
 
         columns = info.id_columns + ('key',)
         query = f"""
-            DELETE FROM {info.table_prefix}metadata
+            DELETE FROM {info.table_prefix}tags
             WHERE (
                 {', '.join(columns)}
             ) = (
@@ -1243,89 +1254,7 @@ class Storage:
 
         with self.db:
             cursor = self.db.execute(query, params)
-        rowcount_exactly_one(
-            cursor, lambda: info.metadata_not_found_exc(*object_id, key=key)
-        )
-
-    @wrap_exceptions(StorageError)
-    def add_tag(self, object_id: Tuple[str, ...], tag: str) -> None:
-        info = SCHEMA_INFO[len(object_id)]
-
-        columns = info.id_columns + ('tag',)
-        query = f"""
-            INSERT INTO {info.table_prefix}tags (
-                {', '.join(columns)}
-            ) VALUES (
-                {', '.join(('?' for _ in columns))}
-            )
-        """
-        params = object_id + (tag,)
-
-        with self.db:
-            try:
-                self.db.execute(query, params)
-            except sqlite3.IntegrityError as e:
-                if "foreign key constraint failed" in str(e).lower():
-                    raise info.not_found_exc(*object_id) from None
-                # tag exists is a no-op; it looks like:
-                # "UNIQUE constraint failed: feed_tags.feed, feed_tags.tag"
-
-    @wrap_exceptions(StorageError)
-    def remove_tag(self, object_id: Tuple[str, ...], tag: str) -> None:
-        info = SCHEMA_INFO[len(object_id)]
-
-        columns = info.id_columns + ('tag',)
-        query = f"""
-            DELETE FROM {info.table_prefix}tags
-            WHERE (
-                {', '.join(columns)}
-            ) = (
-                {', '.join(('?' for _ in columns))}
-            )
-        """
-        params = object_id + (tag,)
-
-        with self.db:
-            self.db.execute(query, params)
-
-    # Tuple[Optional[str], ...] seems hacky,
-    # but we can't have Optional[Tuple[str, ...]]
-    # because we depend on the tuple length
-    # to distinguish between feeds and entries.
-
-    def get_tags(self, object_id: Tuple[Optional[str], ...]) -> Iterable[str]:
-        yield from join_paginated_iter(
-            partial(self.get_tags_page, object_id),
-            self.chunk_size,
-        )
-
-    @wrap_exceptions_iter(StorageError)
-    def get_tags_page(
-        self,
-        object_id: Tuple[Optional[str], ...],
-        chunk_size: Optional[int] = None,
-        last: Optional[_T] = None,
-    ) -> Iterable[Tuple[str, Optional[_T]]]:
-        info = SCHEMA_INFO[len(object_id)]
-
-        query = Query().SELECT_DISTINCT("tag").FROM(f"{info.table_prefix}tags")
-        context: Dict[str, Any] = dict()
-
-        if not any(p is None for p in object_id):
-            for column in info.id_columns:
-                query.WHERE(f"{column} = :{column}")
-            context.update(zip(info.id_columns, object_id))
-
-        query.scrolling_window_order_by("tag")
-
-        def row_factory(t: Tuple[Any, ...]) -> str:
-            tag = t[0]
-            assert isinstance(tag, str)
-            return tag
-
-        yield from paginated_query(
-            self.db, query, context, chunk_size, last, row_factory
-        )
+        rowcount_exactly_one(cursor, lambda: TagNotFoundError(key, cast(str, None)))
 
 
 def make_get_feeds_query(
@@ -1382,7 +1311,7 @@ def apply_feed_filter_options(
     query: Query,
     filter_options: FeedFilterOptions,
 ) -> Dict[str, Any]:
-    url, tags, broken, updates_enabled = filter_options
+    url, tags, broken, updates_enabled, new = filter_options
 
     context: Dict[str, object] = {}
 
@@ -1396,6 +1325,8 @@ def apply_feed_filter_options(
         query.WHERE(f"last_exception IS {'NOT' if broken else ''} NULL")
     if updates_enabled is not None:
         query.WHERE(f"{'' if updates_enabled else 'NOT'} updates_enabled")
+    if new is not None:
+        query.WHERE(f"last_updated is {'' if new else 'NOT'} NULL")
 
     return context
 
@@ -1551,13 +1482,13 @@ def apply_feed_tags_filter_options(
         add(str(tag_query))
 
     if add_tags_cte:
-        query.WITH(("tags", f"SELECT tag FROM feed_tags WHERE feed = {url_column}"))
+        query.WITH(("tags", f"SELECT key FROM feed_tags WHERE feed = {url_column}"))
 
     if add_tags_count_cte:
         query.WITH(
             (
                 "tags_count",
-                f"SELECT count(tag) FROM feed_tags WHERE feed = {url_column}",
+                f"SELECT count(key) FROM feed_tags WHERE feed = {url_column}",
             )
         )
 
@@ -1725,3 +1656,22 @@ def make_entry_counts_query(
         query.SELECT(f"(SELECT count(*) / :{days_param} FROM kfu_{period_i})")
 
     return query, context
+
+
+def entry_update_intent_to_dict(intent: EntryUpdateIntent) -> Mapping[str, Any]:
+    context = intent._asdict()
+    entry = context.pop('entry')
+    context.update(
+        entry._asdict(),
+        content=(
+            json.dumps([t._asdict() for t in entry.content]) if entry.content else None
+        ),
+        enclosures=(
+            json.dumps([t._asdict() for t in entry.enclosures])
+            if entry.enclosures
+            else None
+        ),
+        data_hash=entry.hash,
+        data_hash_changed=context.pop('hash_changed'),
+    )
+    return context
