@@ -1,6 +1,7 @@
 from dataclasses import dataclass
 from datetime import datetime
 from datetime import timezone
+from functools import cached_property
 from types import MappingProxyType
 from types import SimpleNamespace
 from typing import Any
@@ -15,7 +16,6 @@ from typing import TypeVar
 from typing import Union
 
 from ._hash_utils import get_hash
-from ._vendor.functools import cached_property
 from .types import _entry_argument
 from .types import _feed_argument
 from .types import _namedtuple_compat
@@ -209,6 +209,7 @@ class ParsedFeed(NamedTuple):
     entries: Iterable[EntryData]
     http_etag: Optional[str] = None
     http_last_modified: Optional[str] = None
+    mime_type: Optional[str] = None
 
 
 class FeedForUpdate(NamedTuple):
@@ -416,6 +417,7 @@ class FeedFilterOptions(NamedTuple):
     tags: TagFilter = ()
     broken: Optional[bool] = None
     updates_enabled: Optional[bool] = None
+    new: Optional[bool] = None
 
     @classmethod
     def from_args(
@@ -424,6 +426,7 @@ class FeedFilterOptions(NamedTuple):
         tags: TagFilterInput = None,
         broken: Optional[bool] = None,
         updates_enabled: Optional[bool] = None,
+        new: Optional[bool] = None,
     ) -> _FFO:
         feed_url = _feed_argument(feed) if feed is not None else None
         tag_filter = tag_filter_argument(tags)
@@ -432,8 +435,10 @@ class FeedFilterOptions(NamedTuple):
             raise ValueError("broken should be one of (None, False, True)")
         if updates_enabled not in (None, False, True):
             raise ValueError("updates_enabled should be one of (None, False, True)")
+        if new not in (None, False, True):
+            raise ValueError("new should be one of (None, False, True)")
 
-        return cls(feed_url, tag_filter, broken, updates_enabled)
+        return cls(feed_url, tag_filter, broken, updates_enabled, new)
 
 
 @dataclass(frozen=True)
