@@ -1,7 +1,7 @@
+from functools import cached_property
 from typing import Any
 from typing import Tuple
-
-from ._vendor.functools import cached_property
+from typing import Union
 
 
 class _FancyExceptionBase(Exception):
@@ -56,6 +56,25 @@ class ReaderError(_FancyExceptionBase):
     """Base for all public exceptions."""
 
 
+class ReaderWarning(UserWarning):
+    """Base for all warnings emitted by *reader*.
+    that are not :exc:`DeprecationWarning`.
+
+    .. versionadded:: 2.13
+
+    """
+
+
+class ResourceNotFoundError(ReaderError):
+    """Resource (feed, entry) not found.
+
+    .. versionadded:: 2.8
+
+    """
+
+    # TODO: object_id: tuple[str, ...] (but FeedError must become tuple[str]!)
+
+
 class FeedError(ReaderError):
     """A feed error occurred."""
 
@@ -85,7 +104,7 @@ class FeedExistsError(FeedError):
     message = "feed exists"
 
 
-class FeedNotFoundError(FeedError):
+class FeedNotFoundError(FeedError, ResourceNotFoundError):
     """Feed not found."""
 
     message = "no such feed"
@@ -101,7 +120,7 @@ class InvalidFeedURLError(FeedError, ValueError):
     message = "invalid feed URL"
 
 
-class ParseError(FeedError):
+class ParseError(FeedError, ReaderWarning):
     """An error occurred while getting/parsing feed.
 
     The original exception should be chained to this one (e.__cause__).
@@ -149,7 +168,7 @@ class EntryExistsError(EntryError):
     message = "entry exists"
 
 
-class EntryNotFoundError(EntryError):
+class EntryNotFoundError(EntryError, ResourceNotFoundError):
     """Entry not found."""
 
     message = "no such entry"
@@ -163,6 +182,9 @@ class MetadataError(ReaderError):
         Signature changed from ``MetadataError(message='')``
         to ``MetadataError(key, message='')``.
 
+    .. deprecated:: 2.8
+        Will be removed in *reader* 3.0.
+
     """
 
     def __init__(self, *args: Any, key: str, **kwargs: Any) -> None:
@@ -184,6 +206,9 @@ class MetadataNotFoundError(MetadataError):
         Signature changed from ``MetadataNotFoundError(url, key, message='')``
         to ``MetadataNotFoundError(key, message='')``.
 
+    .. deprecated:: 2.8
+        Will be removed in *reader* 3.0.
+
     """
 
     message = "no such metadata"
@@ -194,6 +219,9 @@ class FeedMetadataNotFoundError(MetadataNotFoundError, FeedError):
 
     .. versionadded:: 1.18
 
+    .. deprecated:: 2.8
+        Will be removed in *reader* 3.0.
+
     """
 
     def __init__(self, url: str, key: str, message: str = '') -> None:
@@ -205,6 +233,9 @@ class EntryMetadataNotFoundError(MetadataNotFoundError, EntryError):
 
     .. versionadded:: 1.18
 
+    .. deprecated:: 2.8
+        Will be removed in *reader* 3.0.
+
     """
 
     def __init__(self, feed_url: str, id: str, key: str, message: str = '') -> None:
@@ -248,3 +279,41 @@ class InvalidPluginError(PluginError, ValueError):
     .. versionadded:: 1.16
 
     """
+
+
+class TagError(ReaderError):
+    """A tag error occurred.
+
+    .. versionadded:: 2.8
+
+    """
+
+    def __init__(
+        self,
+        key: str,
+        object_id: Union[Tuple[()], str, Tuple[str, str]],
+        message: str = '',
+    ) -> None:
+        super().__init__(message)
+
+        #: The tag key.
+        self.key = key
+
+        # TODO: tuple[str, ...], once FeedError.object_id becomes tuple[str]
+
+        #: The `object_id` of the resource.
+        self.object_id = object_id
+
+    @property
+    def _str(self) -> str:
+        return f"{self.object_id!r}: {self.key!r}"
+
+
+class TagNotFoundError(TagError):
+    """Tag not found.
+
+    .. versionadded:: 2.8
+
+    """
+
+    message = "no such tag"
