@@ -11,19 +11,19 @@ from typing import cast
 from typing import Dict
 from typing import Iterable
 from typing import List
+from typing import Literal
 from typing import Mapping
 from typing import NamedTuple
 from typing import Optional
+from typing import overload
+from typing import Protocol
+from typing import runtime_checkable
 from typing import Sequence
 from typing import Tuple
 from typing import Type
 from typing import TypeVar
 from typing import Union
 
-from typing_extensions import Literal
-from typing_extensions import Protocol
-from typing_extensions import runtime_checkable
-
 from reader.exceptions import ReaderError
 
 
@@ -329,6 +329,22 @@ class Entry(_namedtuple_compat):
         """
         return self.updated or self.added
 
+    def get_content(self, *, prefer_summary: bool = False) -> Optional['Content']:
+        """Return a text content OR the summary.
+
+        Prefer HTML content, when available.
+
+        Args:
+            prefer_summary (bool): Return summary, if available.
+
+        Returns:
+            Content or none: The content, if found.
+
+        .. versionadded:: 2.12
+
+        """
+        return _get_entry_content(self, prefer_summary)
+
 
 @dataclass(frozen=True)
 class Content(_namedtuple_compat):
@@ -346,6 +362,41 @@ class Content(_namedtuple_compat):
     #: The content language.
     language: Optional[str] = None
 
+    @property
+    def is_html(self) -> bool:
+        """Whether the content is (X)HTML.
+
+        True if the content does not have a type.
+
+        .. versionadded:: 2.12
+        """
+        if self.type:
+            return self.type in _HTML_CONTENT_TYPES
+        return True
+
+
+_PREFERRED_CONTENT_TYPES = ['text/html', 'text/xhtml', 'text/plain', None]
+_HTML_CONTENT_TYPES = {'text/html', 'text/xhtml'}
+_HTML_CONTENT_TYPE = 'text/html'
+
+
+def _get_entry_content(entry: Entry, prefer_summary: bool = False) -> Optional[Content]:
+    # TODO: Make this public; Entry should be a protocol, Content should be generic.
+    # TODO: Use the type from .summary_detail (when we get it).
+
+    if prefer_summary and entry.summary:
+        return Content(entry.summary)
+
+    for type in _PREFERRED_CONTENT_TYPES:
+        for content in entry.content:
+            if content.type == type and content.value:
+                return content
+
+    if entry.summary:
+        return Content(entry.summary)
+
+    return None
+
 
 @dataclass(frozen=True)
 class Enclosure(_namedtuple_compat):
@@ -608,6 +659,7 @@ SearchSortOrder = Literal['relevant', 'recent', 'random']
 
 
 # https://github.com/python/typing/issues/182
+# TODO: allow JSONType to be str, int, ...
 JSONValue = Union[str, int, float, bool, None, Dict[str, Any], List[Any]]
 JSONType = Union[Dict[str, JSONValue], List[JSONValue]]
 
@@ -638,28 +690,71 @@ class EntryLike(Protocol):
         ...
 
 
+# https://github.com/lemon24/reader/issues/266#issuecomment-1013739526
+GlobalInput = Tuple[()]
 FeedInput = Union[str, FeedLike]
 EntryInput = Union[Tuple[str, str], EntryLike]
+ResourceInput = Union[GlobalInput, FeedInput, EntryInput]
+AnyResourceInput = Union[ResourceInput, None, Tuple[None], Tuple[None, None]]
+ResourceId = Union[Tuple[()], Tuple[str], Tuple[str, str]]
+AnyResourceId = Union[ResourceId, None, Tuple[None], Tuple[None, None]]
 
 
 def _feed_argument(feed: FeedInput) -> str:
     if isinstance(feed, FeedLike):
-        return feed.url
-    if isinstance(feed, str):
-        return feed
+        rv = feed.url
+    elif isinstance(feed, tuple) and len(feed) == 1:
+        rv = feed[0]
+    else:
+        rv = feed
+    if isinstance(rv, str):
+        return rv
     raise ValueError(f'invalid feed argument: {feed!r}')
 
 
 def _entry_argument(entry: EntryInput) -> Tuple[str, str]:
     if isinstance(entry, EntryLike):
-        return _feed_argument(entry.feed_url), entry.id
-    if isinstance(entry, tuple) and len(entry) == 2:
-        feed_url, entry_id = entry
+        rv = _feed_argument(entry.feed_url), entry.id
+    elif isinstance(entry, tuple) and len(entry) == 2:
+        rv = entry
+    else:
+        rv = None
+    if rv:
+        feed_url, entry_id = rv
         if isinstance(feed_url, str) and isinstance(entry_id, str):
-            return entry
+            return rv
     raise ValueError(f'invalid entry argument: {entry!r}')
 
 
+@overload
+def _resource_argument(resource: GlobalInput) -> Tuple[()]:
+    ...  # pragma: no cover
+
+
+@overload
+def _resource_argument(resource: FeedInput) -> Tuple[str]:
+    ...  # pragma: no cover
+
+
+@overload
+def _resource_argument(resource: EntryInput) -> Tuple[str, str]:
+    ...  # pragma: no cover
+
+
+def _resource_argument(resource: ResourceInput) -> ResourceId:
+    if isinstance(resource, tuple) and len(resource) == 0:
+        return resource
+    try:
+        return (_feed_argument(resource),)  # type: ignore[arg-type]
+    except ValueError:
+        pass
+    try:
+        return _entry_argument(resource)  # type: ignore[arg-type]
+    except ValueError:
+        pass
+    raise ValueError(f"invalid resource argument: {resource!r}")
+
+
 # str explicitly excluded, to allow for a string-based query language;
 # https://github.com/lemon24/reader/issues/184#issuecomment-689587006
 TagFilterInput = Union[
