@@ -4,13 +4,13 @@ import logging
 from contextlib import contextmanager
 from unittest.mock import MagicMock
 
-import feedparser
 import pytest
 import requests
 from utils import make_url_base
 
 from reader import Feed
 from reader._parser import default_parser
+from reader._parser import FeedArgumentTuple
 from reader._parser import FeedparserParser
 from reader._parser import FileRetriever
 from reader._parser import JSONFeedParser
@@ -18,6 +18,7 @@ from reader._parser import Parser
 from reader._parser import RetrieveResult
 from reader._parser import SessionWrapper
 from reader._types import FeedData
+from reader._vendor import feedparser
 from reader.exceptions import ParseError
 
 
@@ -171,7 +172,7 @@ def test_parse(monkeypatch, feed_type, data_file, parse, make_url, data_dir):
     expected = {'url_base': url_base, 'rel_base': rel_base}
     exec(data_dir.join(feed_filename + '.py').read(), expected)
 
-    feed, entries, _, _ = parse(feed_url)
+    feed, entries, _, _, mime_type = parse(feed_url)
     entries = list(entries)
 
     assert feed == expected['feed']
@@ -191,7 +192,7 @@ def test_no_mime_type(monkeypatch, parse, make_url, data_dir):
 
     parse.mount_parser_by_url(feed_url, custom_parser)
 
-    feed, entries, _, _ = parse(feed_url)
+    feed, entries, _, _, mime_type = parse(feed_url)
 
     with open(str(feed_path), encoding='utf-8') as f:
         expected_feed = FeedData(url=feed_url, title=f.read())
@@ -353,18 +354,18 @@ def test_parse_returns_etag_last_modified(
     make_http_etag_last_modified_url.last_modified = 'last_modified'
 
     feed_url = make_http_etag_last_modified_url(data_dir.join('full.' + feed_type))
-    _, _, etag, last_modified = parse(feed_url)
+    _, _, etag, last_modified, _ = parse(feed_url)
 
     assert etag == 'etag'
     assert last_modified == 'last_modified'
 
     feed_url = make_http_url(data_dir.join('full.atom'))
-    _, _, etag, last_modified = parse(feed_url)
+    _, _, etag, last_modified, _ = parse(feed_url)
 
     assert etag == last_modified == None
 
     feed_url = make_relative_path_url(data_dir.join('full.' + feed_type))
-    _, _, etag, last_modified = parse(feed_url)
+    _, _, etag, last_modified, _ = parse(feed_url)
 
     assert etag == last_modified == None
 
@@ -382,7 +383,7 @@ def test_parse_local_timezone(monkeypatch_tz, request, parse, tz, data_dir):
     exec(feed_path.new(ext='.atom.py').read(), expected)
 
     monkeypatch_tz(tz)
-    feed, _, _, _ = parse(str(feed_path))
+    feed, _, _, _, _ = parse(str(feed_path))
     assert feed.updated == expected['feed'].updated
 
 
@@ -413,7 +414,7 @@ def test_parse_response_plugins(monkeypatch, tmpdir, make_http_url, data_dir):
     parse.session_hooks.response.append(do_nothing_plugin)
     parse.session_hooks.response.append(rewrite_to_empty_plugin)
 
-    feed, _, _, _ = parse(feed_url)
+    feed, _, _, _, _ = parse(feed_url)
     assert req_plugin.called
     assert do_nothing_plugin.called
     assert rewrite_to_empty_plugin.called
@@ -455,6 +456,24 @@ def test_user_agent_none(parse, make_http_get_headers_url, data_dir):
     assert 'reader' not in headers['User-Agent']
 
 
+def test_parallel_persistent_session(parse, make_http_url, data_dir):
+    sessions = []
+
+    def req_plugin(session, request, **kwargs):
+        sessions.append(session)
+
+    parse.session_hooks.request.append(req_plugin)
+
+    feeds = [
+        FeedArgumentTuple(make_http_url(data_dir.join(name)))
+        for name in ('empty.atom', 'empty.rss')
+    ]
+    list(parse.parallel(feeds))
+
+    assert len(sessions) == 2
+    assert sessions[0] is sessions[1]
+
+
 @pytest.mark.parametrize('exc_cls', [Exception, OSError])
 def test_feedparser_parse_call(monkeypatch, parse, make_url, data_dir, exc_cls):
     """feedparser.parse must always be called with True
@@ -510,7 +529,7 @@ def test_missing_entry_id():
     assert entry.id == entry.link
 
     # ... and only link.
-    with pytest.raises(ParseError) as excinfo:
+    with pytest.raises(ParseError) as excinfo, pytest.warns(ParseError):
         feedparser_parse(
             'url',
             """
@@ -529,8 +548,35 @@ def test_missing_entry_id():
     assert excinfo.value.url == 'url'
     assert 'entry with no id' in excinfo.value.message
 
+    # But we only raise an exception if *all* entries fail, warn otherwise.
+    with pytest.warns(ParseError) as warnings:
+        feed, entries = feedparser_parse(
+            'url',
+            """
+            <?xml version="1.0" encoding="UTF-8" ?>
+            <rss version="2.0">
+            <channel>
+                <item>
+                    <title>Example entry without id or link</title>
+                </item>
+                <item>
+                    <title>Another one</title>
+                </item>
+                <item>
+                    <link>http://www.example.com/blog/post/1</link>
+                    <title>Example entry</title>
+                </item>
+            </channel>
+            </rss>
+            """.strip(),
+        )
+    assert [e.title for e in entries] == ["Example entry"]
+    assert len(warnings) == 2, warnings
+    assert warnings[0].message.url == 'url'
+    assert 'entry with no id' in warnings[0].message.message
+
     # There is no fallback for Atom.
-    with pytest.raises(ParseError) as excinfo:
+    with pytest.raises(ParseError) as excinfo, pytest.warns(ParseError):
         feedparser_parse(
             'url',
             """
@@ -799,7 +845,7 @@ def test_feed_root_nonenmpty_bad_paths(data_dir, url, reason):
 
 
 BAD_PATHS_WINDOWS_BY_REASON = [
-    ('device file', ['NUL', 'CON']),
+    ('path must not be reserved', ['NUL', 'CON']),
     (
         'path must be relative',
         [
@@ -868,6 +914,7 @@ def make_dummy_retriever(name, mime_type='type/subtype', headers=None):
         retriever.last_http_accept = http_accept
         yield RetrieveResult(name, mime_type, http_etag, http_last_modified, headers)
 
+    retriever.slow_to_read = False
     return retriever
 
 
@@ -900,6 +947,7 @@ def test_parser_selection():
         'http:one',
         'etag',
         None,
+        'type/http',
     )
     assert http_retriever.last_http_accept == 'type/http'
     assert http_parser.last_headers == 'headers'
@@ -922,6 +970,7 @@ def test_parser_selection():
         'file:one',
         None,
         'last-modified',
+        'type/file',
     )
     assert file_retriever.last_http_accept == 'type/http,type/file,text/plain;q=0.8'
     assert file_parser.last_headers is None
@@ -949,13 +998,25 @@ def test_parser_selection():
     assert "unaware parser" in str(excinfo.value)
 
     parse.mount_parser_by_mime_type(make_dummy_parser('fallbackp-'), '*/*')
-    assert parse('nomt:one') == ('fallbackp-nomt', 'nomt:one', None, None)
-    assert parse('unkn:one') == ('fallbackp-unkn', 'unkn:one', None, None)
+    assert parse('nomt:one') == (
+        'fallbackp-nomt',
+        'nomt:one',
+        None,
+        None,
+        'application/octet-stream',
+    )
+    assert parse('unkn:one') == (
+        'fallbackp-unkn',
+        'unkn:one',
+        None,
+        None,
+        'type/unknown',
+    )
     assert nomt_retriever.last_http_accept == 'type/http,type/file,*/*,text/plain;q=0.8'
 
-    assert parse('file:o') == ('urlp-file', 'file:o', None, None)
+    assert parse('file:o') == ('urlp-file', 'file:o', None, None, 'type/file')
     assert file_retriever.last_http_accept is None
-    assert parse('file:///o') == ('urlp-file', 'file:///o', None, None)
+    assert parse('file:///o') == ('urlp-file', 'file:///o', None, None, 'type/file')
 
 
 def test_retriever_selection():
@@ -970,12 +1031,14 @@ def test_retriever_selection():
         'http://generic.com/',
         'etag',
         None,
+        'type/subtype',
     )
     assert parse('http://specific.com/', None, 'last-modified') == (
         'specific',
         'http://specific.com/',
         None,
         'last-modified',
+        'type/subtype',
     )
 
     with pytest.raises(ParseError) as excinfo:
