@@ -254,30 +254,18 @@ def get_entries_chunk_size_1(storage, _, __):
     list(storage.get_entries_page(chunk_size=1, now=datetime(2010, 1, 1)))
 
 
-def iter_metadata(storage, feed, __):
-    list(storage.iter_metadata((feed.url,)))
-
-
-def set_metadata(storage, feed, __):
-    storage.set_metadata((feed.url,), 'key', 'value')
-
-
-def delete_metadata(storage, feed, __):
-    storage.delete_metadata((feed.url,), 'key')
-
-
-def add_tag(storage, feed, __):
-    storage.add_tag((feed.url,), 'tag')
-
-
-def remove_tag(storage, feed, __):
-    storage.remove_tag((feed.url,), 'tag')
-
-
 def get_tags(storage, feed, __):
     list(storage.get_tags((feed.url,)))
 
 
+def set_tag(storage, feed, __):
+    storage.set_tag((feed.url,), 'key', 'value')
+
+
+def delete_tag(storage, feed, __):
+    storage.delete_tag((feed.url,), 'key')
+
+
 def get_feed_counts(storage, _, __):
     storage.get_feed_counts()
 
@@ -317,12 +305,9 @@ def get_entry_last(storage, feed, entry):
         delete_entries,
         get_entries_chunk_size_0,
         get_entries_chunk_size_1,
-        iter_metadata,
-        set_metadata,
-        delete_metadata,
-        add_tag,
-        remove_tag,
         get_tags,
+        set_tag,
+        delete_tag,
         get_feed_counts,
         get_entry_counts,
         get_feed_last,
@@ -403,10 +388,6 @@ def iter_pagination_chunk_size_3(storage):
     return storage.get_entries_page(chunk_size=3, now=datetime(2010, 1, 1))
 
 
-def iter_iter_metadata(storage):
-    return storage.iter_metadata_page(('two',), chunk_size=1)
-
-
 def iter_get_tags(storage):
     return storage.get_tags_page(('two',), chunk_size=1)
 
@@ -424,7 +405,6 @@ def iter_get_tags(storage):
         iter_pagination_chunk_size_1,
         iter_pagination_chunk_size_2,
         iter_pagination_chunk_size_3,
-        iter_iter_metadata,
         iter_get_tags,
     ],
 )
@@ -459,10 +439,8 @@ def check_iter_locked(db_path, pre_stuff, iter_stuff):
             0,
         )
     )
-    storage.set_metadata(('two',), '1', 1)
-    storage.set_metadata(('two',), '2', 2)
-    storage.add_tag(('two',), '1')
-    storage.add_tag(('two',), '2')
+    storage.set_tag(('two',), '1', 1)
+    storage.set_tag(('two',), '2', 2)
 
     if pre_stuff:
         pre_stuff(storage)
@@ -483,79 +461,6 @@ def test_update_feed_last_updated_not_found(db_path):
         storage.update_feed(FeedUpdateIntent('inexistent-feed', datetime(2010, 1, 2)))
 
 
-# aviod spurious PyPy "sqlite3.InterfaceError: Error binding parameter X")
-@pytest.mark.skipif("sys.implementation.name == 'pypy'")
-@pytest.mark.parametrize(
-    'entry_count',
-    [
-        # We assume the query uses 2 parameters per entry (feed URL and entry ID).
-        #
-        # variable_number defaults to 999 when compiling SQLite from sources
-        int(999 / 2) + 1,
-        # variable_number defaults to 250000 in Ubuntu 18.04 -provided SQLite
-        pytest.param(
-            int(250000 / 2) + 1,
-            marks=(pytest.mark.slow,),
-        ),
-    ],
-)
-def test_get_entries_for_update_param_limit(entry_count):
-    """get_entries_for_update() should work even if the number of query
-    parameters goes over the variable_number SQLite run-time limit.
-
-    https://github.com/lemon24/reader/issues/109
-
-    """
-    storage = Storage(':memory:')
-
-    # shouldn't raise an exception
-    list(
-        storage.get_entries_for_update(
-            ('feed', 'entry-{}'.format(i)) for i in range(entry_count)
-        )
-    )
-    list(
-        storage.get_entries_for_update(
-            ('feed', 'entry-{}'.format(i)) for i in range(entry_count)
-        )
-    )
-
-
-class StorageNoGetEntriesForUpdateFallback(Storage):
-    def _get_entries_for_update_n_queries(self, _):
-        assert False, "shouldn't get called"
-
-
-class StorageAlwaysGetEntriesForUpdateFallback(Storage):
-    def _get_entries_for_update_one_query(self, _):
-        raise sqlite3.OperationalError("too many SQL variables")
-
-
-@pytest.mark.parametrize(
-    'storage_cls',
-    [StorageNoGetEntriesForUpdateFallback, StorageAlwaysGetEntriesForUpdateFallback],
-)
-def test_get_entries_for_update(storage_cls):
-    storage = storage_cls(':memory:')
-    storage.add_feed('feed', datetime(2010, 1, 1))
-    entry = EntryData('feed', 'one', datetime(2010, 1, 1))
-    storage.add_or_update_entry(
-        EntryUpdateIntent(
-            entry,
-            datetime(2010, 1, 2),
-            datetime(2010, 1, 1),
-            datetime(2010, 1, 1),
-            0,
-            0,
-        )
-    )
-
-    assert list(storage.get_entries_for_update([('feed', 'one'), ('feed', 'two')])) == [
-        EntryForUpdate(datetime(2010, 1, 1), None, entry.hash, 0),
-        None,
-    ]
-
-
 @pytest.fixture
 def storage():
     return Storage(':memory:')
