@@ -5,6 +5,7 @@ from types import SimpleNamespace
 
 import pytest
 
+from reader import Content
 from reader import Entry
 from reader import EntryError
 from reader import EntrySearchResult
@@ -19,6 +20,7 @@ from reader._types import FeedData
 from reader.types import _entry_argument
 from reader.types import _feed_argument
 from reader.types import _namedtuple_compat
+from reader.types import _resource_argument
 from reader.types import MISSING
 
 
@@ -39,12 +41,36 @@ def test_namedtuple_compat():
     assert Object(1, 2)._asdict() == {'one': 1, 'two': 2}
 
 
+BAD_RESOURCE_ARGUMENTS = [
+    1,
+    Feed(None),
+    Feed(1),
+    (1, 'b'),
+    ('a', 2),
+    (None, 'b'),
+    ('a', None),
+    ('a', 'b', 'c'),
+    Entry('entry', feed=None),
+    Entry(None, feed=Feed('url')),
+    Entry(1, feed=Feed('url')),
+    Entry('entry', feed=Feed(None)),
+]
+BAD_FEED_ARGUMENTS = [(), ('a', 'b')] + BAD_RESOURCE_ARGUMENTS
+BAD_ENTRY_ARGUMENTS = [(), 1, 'ab', Feed('url')] + BAD_RESOURCE_ARGUMENTS
+WILDCARD_ARGUMENTS = [None, (None,), (None, None)]
+
+
 def test__feed_argument():
     feed = Feed('url')
     assert _feed_argument(feed) == feed.url
     assert _feed_argument(feed.url) == feed.url
+    assert _resource_argument((feed.url,)) == (feed.url,)
+
+
+@pytest.mark.parametrize('feed', BAD_FEED_ARGUMENTS + WILDCARD_ARGUMENTS)
+def test__feed_argument_valueerror(feed):
     with pytest.raises(ValueError):
-        _feed_argument(1)
+        _feed_argument(feed)
 
 
 def test__entry_argument():
@@ -53,18 +79,30 @@ def test__entry_argument():
     entry_tuple = feed.url, entry.id
     assert _entry_argument(entry) == entry_tuple
     assert _entry_argument(entry_tuple) == entry_tuple
+
+
+@pytest.mark.parametrize('entry', BAD_ENTRY_ARGUMENTS + WILDCARD_ARGUMENTS)
+def test__entry_argument_valueerror(entry):
     with pytest.raises(ValueError):
-        _entry_argument(entry._replace(feed=None))
+        _entry_argument(entry)
+
+
+def test__resource_argument():
+    feed = Feed('url')
+    entry = Entry('entry', 'updated', feed=feed)
+    entry_tuple = feed.url, entry.id
+    assert _resource_argument(()) == ()
+    assert _resource_argument(feed) == (feed.url,)
+    assert _resource_argument(feed.url) == (feed.url,)
+    assert _resource_argument((feed.url,)) == (feed.url,)
+    assert _resource_argument(entry) == entry_tuple
+    assert _resource_argument(entry_tuple) == entry_tuple
+
+
+@pytest.mark.parametrize('resource', BAD_RESOURCE_ARGUMENTS + WILDCARD_ARGUMENTS)
+def test__resource_argument_valueerror(resource):
     with pytest.raises(ValueError):
-        _entry_argument(1)
-    with pytest.raises(ValueError):
-        _entry_argument('ab')
-    with pytest.raises(ValueError):
-        _entry_argument((1, 'b'))
-    with pytest.raises(ValueError):
-        _entry_argument(('a', 2))
-    with pytest.raises(ValueError):
-        _entry_argument(('a', 'b', 'c'))
+        _resource_argument(resource)
 
 
 def test_object_id():
@@ -228,6 +266,64 @@ def test_entry_updated_not_none():
     assert entry.updated_not_none == entry.updated
 
 
+@pytest.mark.parametrize(
+    'entry_kwargs, kwargs, expected',
+    [
+        ({}, {}, None),
+        (dict(summary='summary'), {}, Content('summary')),
+        (dict(content=[Content('content')]), {}, Content('content')),
+        (dict(summary='summary', content=[Content('content')]), {}, Content('content')),
+        (
+            dict(summary='summary', content=[Content('content')]),
+            dict(prefer_summary=False),
+            Content('content'),
+        ),
+        (
+            dict(summary='summary', content=[Content('content')]),
+            dict(prefer_summary=True),
+            Content('summary'),
+        ),
+        (
+            dict(
+                content=[
+                    Content('notype'),
+                    Content('html', type='text/html'),
+                    Content('plain', type='text/plain'),
+                ]
+            ),
+            {},
+            Content('html', type='text/html'),
+        ),
+        (
+            dict(content=[Content('notype'), Content('plain', type='text/plain')]),
+            {},
+            Content('plain', type='text/plain'),
+        ),
+        (
+            dict(content=[Content('notype'), Content('unknown', type='text/unknown')]),
+            {},
+            Content('notype'),
+        ),
+    ],
+)
+def test_entry_get_content(entry_kwargs, kwargs, expected):
+    assert Entry('id', **entry_kwargs).get_content(**kwargs) == expected
+
+
+@pytest.mark.parametrize(
+    'content, expected',
+    [
+        (Content('value'), True),
+        (Content('value', 'text/html'), True),
+        (Content('value', 'text/xhtml'), True),
+        (Content('value', 'text/plain'), False),
+        (Content('value', 'unknown'), False),
+    ],
+)
+def test_content_is_html(content, expected):
+    assert content.is_html == expected
+
+
 def test_update_result_properties():
     feed = UpdatedFeed('url', 0, 1)
     result = UpdateResult('url', feed)
