@@ -1,5 +1,6 @@
 import pytest
 
+from reader._utils import deprecated
 from reader._utils import deprecated_wrapper
 
 # Normally, the stuff in _utils is tested by tests for higher level code,
@@ -7,22 +8,38 @@ from reader._utils import deprecated_wrapper
 
 
 def test_deprecated_wrapper():
-    def func(arg):
+    def new(arg):
         raise ValueError(arg)
 
-    old_func = deprecated_wrapper('old_func', func, '1.0', '2.0')
+    old = deprecated_wrapper('old', new, '1.0', '2.0')
 
+    _check_deprecated(old)
+
+
+def test_deprecated():
+    @deprecated('new', '1.0', '2.0')
+    def old(arg):
+        "docstring"
+        raise ValueError(arg)
+
+    assert '\n\ndocstring\n\n' in old.__doc__
+
+    _check_deprecated(old)
+
+
+def _check_deprecated(old):
     with pytest.raises(ValueError) as excinfo, pytest.deprecated_call() as warnings:
-        old_func('whatever')
+        old('whatever')
 
     assert excinfo.value.args[0] == 'whatever'
 
-    assert old_func.__name__ == 'old_func'
-    assert old_func.__doc__ == (
-        'Deprecated alias for :meth:`func`.\n\n'
+    assert old.__name__ == 'old'
+    assert old.__doc__.startswith('Deprecated alias for :meth:`new`.\n\n')
+    assert old.__doc__.endswith(
+        '\n'
         '.. deprecated:: 1.0\n'
         '    This method will be removed in *reader* 2.0.\n'
-        '    Use :meth:`func` instead.\n\n'
+        '    Use :meth:`new` instead.\n\n'
     )
 
     assert len(warnings.list) == 1
@@ -31,7 +48,7 @@ def test_deprecated_wrapper():
     assert warning.category is DeprecationWarning
     assert (
         str(warning.message)
-        == 'old_func() is deprecated and will be removed in reader 2.0. Use func() instead.'
+        == 'old() is deprecated and will be removed in reader 2.0. Use new() instead.'
     )
 
 
